﻿// STEP 1 - Add reference - NEXTWUSB_dotNET_12B.dll 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
//SETP 2 - Add Using NEXTWUSB_dotNET_12B
using NEXTWUSB_dotNET_12B;

namespace NEXTW_USBSample
{
    class Program
    {
        // ckeckResponse also check CRC (using checkCRCValue())
        static bool checkResponse(uint slaves, uint column, uint expectValue, uint retry)
        {
            uint realValue = 0xFEDCBA98; //inital data
            uint retried;
            for (retried = 0; retried <= retry; retried++)
            {
                NEXTWUSB.ECMUSBRead();

                if (!NEXTWUSB.checkCRCValue())
                    Console.WriteLine("CRC Check Fail");

                if (column == 0)
                    if (slaves == 41)
                        realValue = NEXTWUSB.respData[slaves].CMD;
                    else
                        realValue = (uint)(NEXTWUSB.respData[slaves].CMD & 0xFFFF);
                else if (column == 1)
                {
                    if (slaves == 0)
                        realValue = (uint)(NEXTWUSB.respData[slaves].Parm & 0xFF);
                    else
                        realValue = NEXTWUSB.respData[slaves].Parm;
                }
                else if (column == 2)
                    realValue = NEXTWUSB.respData[slaves].Data1;
                else
                    realValue = NEXTWUSB.respData[slaves].Data2;

                if (realValue == expectValue)
                {
                    // Console.WriteLine("PASS");
                    break;
                }
                else if (retry == retried)
                {
                    Console.WriteLine("Fail,realValue= " + realValue.ToString() + " expectValue= " + expectValue.ToString() + " retry = " + retried.ToString());
                }
                else
                {
                    NEXTWUSB.ClearCmdData();
                    NEXTWUSB.ECMUSBWrite();
                }
            }
            return (realValue == expectValue);
        }
        // generate CRC8 value
        static void generateCRC8()
        {
            int totalSize,size;
            if (NEXTWUSB.DEF_MA_MAX == 42)
            {
                totalSize = 504;
                size = 3;
            }
            else
            {
                totalSize = 512;
                size = 4;
            }
            byte[] bytearr = new byte[totalSize];
            IntPtr ptr = System.Runtime.InteropServices.Marshal.AllocHGlobal(totalSize);
            for (int count = 0; count < NEXTWUSB.DEF_MA_MAX; count++)
            {
                System.Runtime.InteropServices.Marshal.StructureToPtr(NEXTWUSB.cmdData[count], ptr, true);
                System.Runtime.InteropServices.Marshal.Copy(ptr, bytearr, size * count, size);
            }
            bytearr[3] = 0;
            bytearr[3] = CRC8.CRC(bytearr);
            NEXTWUSB.cmdData[0].Parm = (ushort)(bytearr[3] * 256 + bytearr[2]);
            System.Runtime.InteropServices.Marshal.FreeHGlobal(ptr);
        }
        static void Main(string[] args)
        {
            bool retValue, isOpen = false;

            #region Open the USB Port
            retValue = NEXTWUSB.OpenECMUSB();
            if (retValue)
                isOpen = true;
            else
            {
                Console.WriteLine("Open USB Fail");
                return;
            }
            System.Threading.Thread.Sleep(100);
            #endregion
            #region SET_STATE to PreOP
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_STATE;
            NEXTWUSB.cmdData[0].Data1 = NEXTWUSB.STATE_PRE_OP; // set state to pre-OP
            generateCRC8();
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 2, 100);
            #endregion
            #region SET_STATE to OP
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_STATE;
            NEXTWUSB.cmdData[0].Data1 = NEXTWUSB.STATE_OPERATIONAL;// set state to OP
            generateCRC8();
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 8, 1000);// wait for enter to op mode
            #endregion
            #region SET_STATE to Init
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_STATE;
            NEXTWUSB.cmdData[0].Data1 = NEXTWUSB.STATE_INIT;
            generateCRC8();
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 1, 1000);
            #endregion
            #region Close USB
            NEXTWUSB.CloseECMUSB();
            #endregion
        }
    }
}
