﻿// STEP 1 - Add reference - NEXTWUSB_dotNET_12B.dll 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
//SETP 2 - Add Using NEXTWUSB_dotNET_12B
using NEXTWUSB_dotNET_12B;

namespace NEXTW_USBSample
{
    class Program
    {
        static bool checkResponse(uint slaves, uint column, uint expectValue, uint retry)
        {
            uint realValue = 0xFEDCBA98; //inital data
            uint retried;
            for (retried = 0; retried <= retry; retried++)
            {
                NEXTWUSB.ECMUSBRead();

                if (column == 0)
                    if (slaves == 41)
                        realValue = NEXTWUSB.respData[slaves].CMD;
                    else
                        realValue = (uint)(NEXTWUSB.respData[slaves].CMD & 0xFFFF);
                else if (column == 1)
                {
                    if (slaves == 0)
                        realValue = (uint)(NEXTWUSB.respData[slaves].Parm & 0xFF);
                    else
                        realValue = NEXTWUSB.respData[slaves].Parm;
                }
                else if (column == 2)
                    realValue = NEXTWUSB.respData[slaves].Data1;
                else
                    realValue = NEXTWUSB.respData[slaves].Data2;

                if (realValue == expectValue)
                {
                    // Console.WriteLine("PASS");
                    break;
                }
                else if (retry == retried)
                {
                    Console.WriteLine("Fail,realValue= " + realValue.ToString() + " expectValue= " + expectValue.ToString() + " retry = " + retried.ToString());
                }
                else
                {
                    NEXTWUSB.ClearCmdData();
                    NEXTWUSB.ECMUSBWrite();
                }
            }
            return (realValue == expectValue);
        }
        static void Main(string[] args)
        {
            bool retValue, isOpen = false;

            #region STEP 3 - Open the USB Port
            retValue = NEXTWUSB.OpenECMUSB();
            if (retValue)
                isOpen = true;
            else
            {
                Console.WriteLine("Open USB Fail");
                return;
            }
            System.Threading.Thread.Sleep(100);
            #endregion
            #region STEP 4 - SET_STATE to PreOP
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_STATE;
            NEXTWUSB.cmdData[0].Data1 = NEXTWUSB.STATE_PRE_OP; // set state to pre-OP
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 2, 100);
            #endregion
            #region STEP 5 - SET_AXIS
            NEXTWUSB.ClearCmdData();
            char[] slaveType = { 'd', 'i', 'p', 'd', 'd', 's', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n'};
            uint topology;
            int j = 0;
            for (int i = 1; i <= 5; i++)
            {                
                topology = 0;
                for (; j < i * 8; j++)
                {
                    switch (slaveType[j])
                    {
                        case 'd':  // 'd' means drive		
                            break;
                        case 'i': // 'i' means io
                            topology += (uint)(Math.Pow(16, (j % 8)));
                            break;
                        case 'p': // 'p' means HSP(High Speed Pulse)
                            topology += (uint)(Math.Pow(16, (j % 8))*2);
                            break;
						case 's': // 's' means Step
                            topology += (uint)(Math.Pow(16, (j % 8))*3);
                            break;
                        default:
                            break;
                    }
                }
                NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_AXIS;
                NEXTWUSB.cmdData[0].Parm = (ushort)(i - 1); // Group

                NEXTWUSB.cmdData[0].Data1 = topology;
                NEXTWUSB.cmdData[0].Data2 = 0;
                if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                    Console.WriteLine("Write Error");
                System.Threading.Thread.Sleep(2);
            }
            #endregion
            #region STEP 6 - SET_DC
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_DC;
            NEXTWUSB.cmdData[0].Parm = 0;
            NEXTWUSB.cmdData[0].Data1 = 2000; // set cycle time to 2000 us
            NEXTWUSB.cmdData[0].Data2 = 0xFFFF; // Auto offet adjustment

            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(2);
            #endregion
            #region STEP 7 - Set DRIVE_MODE
            NEXTWUSB.ClearCmdData();
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX-1; i++)
            {
                if (slaveType[i - 1] == 'd' || slaveType[i - 1] == 'p' || slaveType[i - 1] == 's')
                {
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.DRIVE_MODE;
                    NEXTWUSB.cmdData[i].Data1 = NEXTWUSB.CSP_MODE; // set to CSP mode
                    NEXTWUSB.cmdData[i].Data2 = NEXTWUSB.DCSYNC; // set using DC sync
                }
            }
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(2);
            #endregion
            #region STEP 7.1 - Read Slave 1 Object 0x1018:01 (Vender ID) via SDO (This step could be skipped)
	        NEXTWUSB.ClearCmdData();
	        NEXTWUSB.cmdData[41].CMD = NEXTWUSB.SDO_RD;
	        NEXTWUSB.cmdData[41].Parm = 1; // slave index (1~40)
	        NEXTWUSB.cmdData[41].Data1 = 0x10180001;// Object Index, subIndex
	        if (!isOpen || !NEXTWUSB.ECMUSBWrite())
		        Console.WriteLine("Write Error");
	        checkResponse(41,0,NEXTWUSB.SDO_RD,1000);  // check Response
	        checkResponse(41,1,1,1000);
	        checkResponse(41,2,0x10180001,1000);
	        NEXTWUSB.ECMUSBRead();
	        Console.WriteLine("Read Slave 1 Vender ID via SDO: " + NEXTWUSB.respData[41].Data2);
            #endregion

            #region STEP 7.2 - Write Slave 1 Object 0x1F01:02 (Sync error count limit) via SDO (This step could be skipped)
	        NEXTWUSB.ClearCmdData();
	        NEXTWUSB.cmdData[41].CMD = NEXTWUSB.SDO_WR;
	        NEXTWUSB.cmdData[41].Parm = 0x0401; // Size(1~4 Byte), slave index (1~40)
	        NEXTWUSB.cmdData[41].Data1 = 0x1F010002;// Object Index, subIndex
	        NEXTWUSB.cmdData[41].Data2 = 9;// Value
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(41, 0, NEXTWUSB.SDO_WR, 1000);  // check Response
            #endregion
            #region STEP 8 - SET_STATE to SafeOP (This step could be skipped)
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_STATE;
            NEXTWUSB.cmdData[0].Data1 = NEXTWUSB.STATE_SAFE_OP;// set state to Safe-OP
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 4, 1000);  // wait for enter to safe op mode
            #endregion
            #region STEP 9 - SET_STATE to OP
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_STATE;
            NEXTWUSB.cmdData[0].Data1 = NEXTWUSB.STATE_OPERATIONAL;// set state to OP
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 8, 1000);// wait for enter to op mode
            #endregion
            #region STEP 10 - SET Servo On
            NEXTWUSB.ClearCmdData();
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX-1; i++)
            {
                if (slaveType[i - 1] == 'd' || slaveType[i - 1] == 'p' || slaveType[i - 1] == 's')  //set drive or HSP slave servo on
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.SV_ON;
            }
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(400); // wait for servo on 
            #endregion

            #region STEP 11- Read Current Position
            NEXTWUSB.ECMUSBRead();
            int[] TargerPosition = new int[NEXTWUSB.DEF_MA_MAX];
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX; i++)
            {
                TargerPosition[i] = (int)NEXTWUSB.respData[i].Data1;
            }
            #endregion

            #region STEP 12- SET Cycle Synchronized Position            
            System.Threading.Thread.Sleep(1000); // wait until motion stop totally
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.ECMUSBRead();
            Console.WriteLine("Current Position: Slave 1 = "+NEXTWUSB.respData[1].Data1.ToString() +
                " Slave 2 = " + NEXTWUSB.respData[2].Data1.ToString() + ", Slave 3 = " + NEXTWUSB.respData[3].Data1.ToString());
            bool showSDOValue = false;
            for (int i = 0; i < 1000; i++) // send 1000 times, let motor move 1000 * 100 pulse
            {
                for (j = 1; j < NEXTWUSB.DEF_MA_MAX-1; j++)
                {
                    if (slaveType[j - 1] == 'd' || slaveType[j - 1] == 'p' || slaveType[j - 1] == 's') //Set drive or HSP slave Cycle Synchronized Position
                    {
                        TargerPosition[j] += 3; // set to move 100 pulse in one cycle time
                        NEXTWUSB.cmdData[j].CMD = NEXTWUSB.CSP;
                        NEXTWUSB.cmdData[j].Data1 = (uint)TargerPosition[j]; 
                    }
                }
                if (i == 100)
                {
                    //Read slave 1 Object 0x60F4:0 (Following Error Actual Value) via SDO
                    NEXTWUSB.cmdData[41].CMD = NEXTWUSB.SDO_RD;
                    NEXTWUSB.cmdData[41].Parm = 1; // slave index (1~40)
                    NEXTWUSB.cmdData[41].Data1 = 0x60F40000;// Object Index, subIndex
                    showSDOValue = true;
                }
                if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                    Console.WriteLine("Write Error");
				if (i % 2 == 0) // USBWrite 2 times and USBRead 1 time
				{
					NEXTWUSB.ECMUSBRead();
                    if ((NEXTWUSB.respData[0].Data2 % 65536) < 80) // If FIFO remaining < 80 then sleep 10ms
                    {
                        if (showSDOValue)
                            if (NEXTWUSB.respData[41].CMD == NEXTWUSB.SDO_RD && NEXTWUSB.respData[41].Parm == 1 && NEXTWUSB.respData[41].Data1 == 0x60F40000)
                            {
                                Console.WriteLine("Read Slave 1 Object 0x60F4:0 (Following Error Value) via SDO: " + NEXTWUSB.respData[41].Data2);
                                showSDOValue = false;
                            }		
                        System.Threading.Thread.Sleep(10);
                    }
				}                
            }
            System.Threading.Thread.Sleep(3000); // wait until motion stop totally
            NEXTWUSB.ECMUSBRead();
            Console.WriteLine("Current Position: Slave 1 = " + NEXTWUSB.respData[1].Data1.ToString() +
                " Slave 2 = " + NEXTWUSB.respData[2].Data1.ToString() + ", Slave 3 = " + NEXTWUSB.respData[3].Data1.ToString());
            #endregion
            #region STEP 13- SET Servo Off
            NEXTWUSB.ClearCmdData();
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX-1; i++)
            {
                if (slaveType[i - 1] == 'd' || slaveType[i - 1] == 'p' || slaveType[i - 1] == 's') //Set drive or HSP slave servo off
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.SV_OFF;
            }
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(100);
            #endregion

            #region STEP 14-Read Input
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.LIO_RD; //ECM can read Local Input
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //only IO slave can read input
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.IO_RD;
            }
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(10);
            NEXTWUSB.ECMUSBRead();
            Console.WriteLine("ECM Local Input Value =" + NEXTWUSB.respData[0].Data1.ToString());
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //IO slave can read input
                    Console.WriteLine("Slave " + i.ToString() + NEXTWUSB.respData[i].Data1.ToString());
                if (slaveType[i - 1] == 'p') //HSP has 8 channel Inputs
                    Console.WriteLine("Slave " + i.ToString() + (NEXTWUSB.respData[i].Data2 & 0xFF).ToString());                 
            } 
            System.Threading.Thread.Sleep(100);
            #endregion

            #region STEP 15-Write Output
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.LIO_WR; //ECM can set Local Output
            NEXTWUSB.cmdData[0].Data1 = 0xF; //Output Value
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //IO slave can write output
                {
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.IO_WR;
                    NEXTWUSB.cmdData[i].Data1 = 0xFFFF; //Output Value
                }
                if (slaveType[i - 1] == 'p')
                {
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.CSP;
                    NEXTWUSB.cmdData[i].Data2 |= 0xFF; //Output Value
                }
            }
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(3000);

            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.LIO_WR; //ECM can set Local Output
            NEXTWUSB.cmdData[0].Data1 = 0xF; //Output Value
            for (int i = 1; i < NEXTWUSB.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //IO slave can write output
                {
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.IO_WR;
                    NEXTWUSB.cmdData[i].Data1 = 0x0;//Output Value
                }
                if (slaveType[i - 1] == 'p')
                {
                    NEXTWUSB.cmdData[i].CMD = NEXTWUSB.CSP;
                    NEXTWUSB.cmdData[i].Data2 &= 0xFF00; //Output Value
                }
            }
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(1000);
            #endregion

            #region SETP 16- SET_STATE to Init
            NEXTWUSB.ClearCmdData();
            NEXTWUSB.cmdData[0].CMD = NEXTWUSB.SET_STATE;
            NEXTWUSB.cmdData[0].Data1 = NEXTWUSB.STATE_INIT;
            if (!isOpen || !NEXTWUSB.ECMUSBWrite())
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 1, 1000);
            #endregion
            #region STEP 17- Close USB
            NEXTWUSB.CloseECMUSB();
            #endregion
        }
    }
}
